#include <vld.h>
#include <iostream>
#include <ctime>
#include <vector>
#include <map>
#include <math.h>
#include <memory>
#include <sstream>
#include <SFML\Graphics.hpp>
#include <SFML\Audio.hpp>
#include <Box2D.h>
using namespace std;
using namespace sf;

string Int2String(int number)
{
   stringstream ss;//create a stringstream
   ss << number;//add number to the stream
   return ss.str();//return a string with the contents of the stream
}

#define GetRandomInt(min,max) rand() % (max-min+1)+min
static const float SCALE = 32.0f;
static const float bj_DEGTORAD = b2_pi/180.0f;
static const float bj_RADTODEG = 180.0f/b2_pi;

namespace Textures
{
enum ID 
{ 
	Planet_1,Planet_2,Planet_3,Planet_4,Planet_5,Planet_6,Planet_7,
	Ship_1,Rocket_1 ,Bomb_1,Laser_1,
	Mineral_1,Mineral_2,Mineral_3,Mineral_4,
	Sky_1,
	Message_1,
	Boom_1,Boom_laser_1,
	Star_light_1,Star_light_2,Star_light_3,Star_light_4,Star_light_5,
	Minimap_1,Minimap_ship_1,Minimap_star_1,
	Ship_turbo_1,Ship_turbo_none,
	Menu_button_1,Menu_screen_1,
	Menu_new_game,Menu_load_game,Menu_options_game,Menu_about_game,Menu_exit_game
};
}
enum GameState
{
	BeforeGame,InGame,PauseGame,OptionsGame,AboutGame,ExitGame
};
enum BoomType
{
	Rocket_1,
	Bomb_1,
	Laser_1
};
enum SoundType
{
	Rocket_blast_1,Bomb_blast_1,Laser_blast_1,
	Rocket_start_1,Bomb_start_1,Laser_start_1,
	Menu_click
};



using namespace Textures;


vector <Sound*> sounds;

class ResManager
{
public:
	map<Textures::ID,unique_ptr<Texture>>TextureMap;
	map<SoundType,unique_ptr<SoundBuffer>>SoundMap;
	Font MainFont;

void LoadFont(const string &Name)
{
	MainFont.loadFromFile(Name);
}

void Load(Textures::ID id, const string& filename)
{
	unique_ptr<Texture> texture(new Texture());
	texture->loadFromFile(filename);
	if (id==Textures::ID::Sky_1)
	{
		texture->setRepeated(true);
	}
	TextureMap.insert(make_pair(id, move(texture)));
}

Texture& Get(Textures::ID id)
{
auto found = TextureMap.find(id);
return *found->second;
}

void LoadSound(SoundType id, const string& filename)
{
	unique_ptr<SoundBuffer> soundBuffer(new SoundBuffer());
	soundBuffer->loadFromFile(filename);
	SoundMap.insert(make_pair(id, move(soundBuffer)));
}

SoundBuffer& GetSound(SoundType id)
{
auto found = SoundMap.find(id);
return *found->second;
}


};

class PlanetDecription
{
public:
	Sprite sprite;
	Text text;
	bool toShow;
	PlanetDecription(ResManager &resManager,Textures::ID id)
{
	toShow=false;
	sprite.setTexture(resManager.Get(id));
	sprite.setPosition(1024-sprite.getLocalBounds().width,768-sprite.getLocalBounds().height);
	text.setFont(resManager.MainFont);
	text.setCharacterSize(16);
	text.setColor(Color::White);
	text.setPosition(sprite.getPosition()+Vector2f(10,10));
	text.setString("ololo");
}
};

class Animation
{
public:
	Sprite sprite;
	int framesCount;
	bool looped;
	vector <IntRect> frames;
	int timeBetweenFrames;
	Vector2i frameSize;
	int lastTime;
	int currentFrame;

	Animation(ResManager &resManager,Textures::ID id,Vector2i _frameSize,int _framesCount,bool _looped,int _timeBetweenFrames)
{
	frameSize=_frameSize;
	framesCount=_framesCount;
	looped=_looped;
	timeBetweenFrames=_timeBetweenFrames;
	lastTime=0;
	currentFrame=0;

	sprite.setTexture(resManager.Get(id));
	sprite.setTextureRect(IntRect(0,0,frameSize.x,frameSize.y));
	
	for (unsigned int i=0;i<framesCount;i++)
	{
		frames.push_back(IntRect(i*frameSize.x,0,frameSize.x,frameSize.y));
	}

}
void Update(int time)
{
	lastTime=lastTime+time;
	if (lastTime>=timeBetweenFrames)
	{
		currentFrame=currentFrame+1;
		if (currentFrame>framesCount-1)
		{
			if (looped==true)
			{
				currentFrame=0;
			}
			else
			{
				currentFrame=framesCount-1;
			}
		}
		lastTime=0;
		sprite.setTextureRect(frames[currentFrame]);

	}
}
};


//b2World world;
class b2Box
{
public:
	b2BodyDef bodyDef;   
    b2Body* body;
    b2PolygonShape shape;
	b2FixtureDef fixtureDef;
};

class b2Circle
{
public:
	b2BodyDef bodyDef;   
    b2Body* body;
    b2CircleShape shape;
	b2FixtureDef fixtureDef;
};

class Bullet:public b2Box
{
	public:
	int type;
	int life;
	bool toDestroy;
	Sprite sprite;
	BoomType boomType;
	Bullet(){}
Bullet(ResManager &resManager,Textures::ID id,b2World &world,int x,int y,float angle,float force,float mass,int _life,BoomType _boomType)
{
	boomType=_boomType;
	type=1;
	life=_life;
	toDestroy=false;

	sprite.setTexture(resManager.Get(id));
	sprite.setOrigin(sprite.getLocalBounds().width/2,sprite.getLocalBounds().height/2);
	sprite.setPosition(x,y);
	sprite.setRotation(angle*bj_RADTODEG+90.0f);


	bodyDef.type = b2_dynamicBody;
	bodyDef.position=b2Vec2(x/SCALE, y/SCALE);
	bodyDef.fixedRotation = true;
	bodyDef.bullet = true;


    body = world.CreateBody(&bodyDef);
	body->SetUserData((void*)type);
	shape.SetAsBox(sprite.getLocalBounds().width/2/SCALE,sprite.getLocalBounds().height/2/SCALE);
	fixtureDef.shape = &shape;
    fixtureDef.density = 1.0f;
    fixtureDef.friction = 1.0f;
    fixtureDef.restitution = 0.5f;
	fixtureDef.userData=(void*)this;
    body->CreateFixture(&fixtureDef);

	body->SetLinearVelocity(b2Vec2(force*cos(angle),force*sin(angle)));
	
	b2MassData *m=new b2MassData;
	body->GetMassData(m);
	m->mass=m->mass*mass;
	body->SetMassData(m);
}
	
};
class Ship:public b2Box
{
public:
	int type;
	int direction;
	float angle;
	Vector2f oldPosition;
	Sprite sprite;
	int mineralCount;
	Text text;
    
Ship(){}

void Create(ResManager &resManager,Textures::ID id,b2World &world,int x,int y,float _angle)
{
	type=2;
	mineralCount=0;
	text.setFont(resManager.MainFont);
	text.setCharacterSize(24);
	text.setColor(Color::Green);
	text.setPosition(20,30);
	text.setString(" : "+Int2String(mineralCount));

	angle=_angle;
	direction=0;
	sprite.setTexture(resManager.Get(id));
	sprite.setRotation(angle*bj_RADTODEG);
	sprite.setOrigin(sprite.getLocalBounds().width/2,sprite.getLocalBounds().height/2);
	sprite.setPosition(x,y);


	bodyDef.type = b2_dynamicBody;
	bodyDef.position.Set((x + sprite.getLocalBounds().width / 2)/SCALE,(y + sprite.getLocalBounds().height / 2)/SCALE),
	bodyDef.fixedRotation = false;

	bodyDef.userData=(void*)type;

    body = world.CreateBody(&bodyDef);
	body->SetUserData((void*) type);
	shape.SetAsBox(sprite.getLocalBounds().width/2/SCALE,sprite.getLocalBounds().height/2/SCALE);
	fixtureDef.shape = &shape;
    fixtureDef.density = 0.0f;
    fixtureDef.friction = 1.0f;
    fixtureDef.restitution = 0.5f;
	fixtureDef.userData=this;
    body->CreateFixture(&fixtureDef);
	b2MassData *m=new b2MassData;
	body->GetMassData(m);
	m->mass=m->mass*5.0f;
	body->SetMassData(m);
	body->SetLinearDamping(0.0f);
}

};

class Planet:public b2Circle
{
public:
	int type;
	Sprite sprite;  
	string description;
Planet(ResManager &resManager,Textures::ID id,b2World &world,int x,int y,const std::string &s)
{
	type=3;
	description=s;
	sprite.setTexture(resManager.Get(id));
	sprite.setOrigin(sprite.getLocalBounds().width/2,sprite.getLocalBounds().height/2);
	sprite.setPosition(x,y);



	bodyDef.type = b2_staticBody;
	bodyDef.position=b2Vec2(x/SCALE, y/SCALE);
	bodyDef.fixedRotation = true;

	bodyDef.userData=(void*)type;

    body = world.CreateBody(&bodyDef);
	body->SetUserData((void*) type);
	shape.m_radius=sprite.getLocalBounds().width/2/SCALE;
	fixtureDef.shape = &shape;
    fixtureDef.density = 1.0f;
    fixtureDef.friction = 1.0f;
    fixtureDef.restitution = 0.5f;
	fixtureDef.userData=this;
    body->CreateFixture(&fixtureDef);
	
}
};

class Mineral:public b2Circle
{
public:
	int type;
	Sprite sprite;  
	bool toDestroy;

Mineral(ResManager &resManager,Textures::ID id,b2World &world,int x,int y)
{
	type=4;
	toDestroy=false;

	sprite.setTexture(resManager.Get(id));
	sprite.setOrigin(sprite.getLocalBounds().width/2,sprite.getLocalBounds().height/2);
	sprite.setPosition(x,y);



	bodyDef.type = b2_dynamicBody;
	bodyDef.position=b2Vec2(x/SCALE, y/SCALE);
	bodyDef.fixedRotation = false;

	bodyDef.userData=(void*)type;

    body = world.CreateBody(&bodyDef);
	body->SetUserData((void*) type);
	shape.m_radius=sprite.getLocalBounds().width/2/SCALE;
	fixtureDef.shape = &shape;
    fixtureDef.density = 1.0f;
    fixtureDef.friction = 1.0f;
    fixtureDef.restitution = 0.5f;
	fixtureDef.userData=this;
    body->CreateFixture(&fixtureDef);
	
	b2MassData *m=new b2MassData;
	body->GetMassData(m);
	m->mass=m->mass*1.5f;
	body->SetMassData(m);
	body->SetLinearDamping(0.67f);
	body->SetAngularDamping(0.8f);

	
}
};
class Sky
{
public:
	Sprite sprite;

Sky(ResManager &resManager,Textures::ID id)
{
	sprite.setOrigin(1024/2,768/2);
	sprite.setTexture(resManager.Get(id));
	sprite.setTextureRect(sf::IntRect(0, 0, 1024, 768));
}
};

class Turbo
{
public:
	Sprite sprite;
	int life;

Turbo(ResManager &resManager,Textures::ID id,int _life,int x,int y,float angle)
{
	sprite.setTexture(resManager.Get(id));
	sprite.setOrigin(sprite.getLocalBounds().width/2,sprite.getLocalBounds().height/2);
	sprite.setPosition(x,y);
	sprite.setRotation(angle*bj_RADTODEG+90.0f);
	life=_life;
}
};


int xx,yy;
class StarLight
{
public:
	Animation *anim;

StarLight(ResManager &resManager,Textures::ID id,Vector2f p)
{
	anim=new Animation(resManager,id,Vector2i(16,16),8,true,250);
	anim->sprite.setOrigin(anim->sprite.getLocalBounds().width/2,anim->sprite.getLocalBounds().height/2);
	anim->sprite.setPosition(p);
}

~StarLight()
{
	delete anim;
}

};

class Boom
{
	public:
	Animation *anim;
	int life;
	BoomType type;

Boom(ResManager &resManager,Vector2f p,int l,BoomType _type)
{	
	Textures::ID id;
	type=_type;
	if (type==BoomType::Rocket_1)
	{
		id=Textures::ID::Boom_1;
		Sound* s_laser=new Sound(resManager.GetSound(SoundType::Rocket_blast_1));
		s_laser->setVolume(50);
		s_laser->play();
		sounds.push_back(s_laser);
	}
	else if (type==BoomType::Bomb_1)
	{
		id=Textures::ID::Boom_1;
		Sound* s_laser=new Sound(resManager.GetSound(SoundType::Bomb_blast_1));
		s_laser->setVolume(50);
		s_laser->play();
		sounds.push_back(s_laser);
	}
	else if (type==BoomType::Laser_1)
	{
		id=Textures::ID::Boom_laser_1;
		Sound* s_laser=new Sound(resManager.GetSound(SoundType::Laser_blast_1));
		s_laser->setVolume(50);
		s_laser->play();
		sounds.push_back(s_laser);
	}
	anim=new Animation(resManager,id,Vector2i(32,32),8,false,50);
	anim->sprite.setOrigin(anim->sprite.getLocalBounds().width/2,anim->sprite.getLocalBounds().height/2);
	anim->sprite.setPosition(p);
	life=l;
}
~Boom()
{
	delete anim;
}
bool Update(int delta)
{
	life=life-delta;
	return (life<=0);
}
};

class FloatingText
{
public:
	Text text;
	std::string string;
	int life;
	int maxLife;

FloatingText(ResManager &resManager,const std::string &s,Vector2f p,int l)
{
	text.setFont(resManager.MainFont);	
	text.setColor(Color::Green);
	text.setCharacterSize(16);
	text.setString(s);
	text.setOrigin(text.getLocalBounds().width/2,text.getLocalBounds().height/2);
	text.setPosition(p);
	life=l;
	maxLife=l;
}

bool Update()
{
	//cout<<"life:"<<life<<endl;
	life=life-1;
	if (life==0)
	{
		return true;
	}
	//cout<<"posx: "<<text.getPosition().x<<" y: "<<text.getPosition().y<<endl;
	//text.setPosition(text.getPosition().x,text.getPosition().y-2);
	return false;
}
};

vector <FloatingText*> floatingText;
vector <Boom*> booms;
ResManager resManager;

class Minimap
{
public:
	Sprite sprite;
	vector <Sprite*> ships;
	vector <Sprite*> stars;
Minimap(ResManager &resManager,Textures::ID id)
{
	sprite.setTexture(resManager.Get(id));
	sprite.setPosition(7,634-128);
}
void Update(ResManager &resManager,Ship &player,vector <Planet*> &s)
{
	for (unsigned int i=0;i<ships.size();i++)
	{
		delete ships[i];
		ships.erase(ships.begin()+i);
	}
	for (unsigned int i=0;i<stars.size();i++)
	{
		delete stars[i];
		stars.erase(stars.begin()+i);
	}
	Sprite* ship=new Sprite();
	ship->setTexture(resManager.Get(Textures::ID::Minimap_ship_1));

	Vector2f p;
	p.x=sprite.getPosition().x+sprite.getLocalBounds().width/2;
	p.y=sprite.getPosition().y+sprite.getLocalBounds().height/2;
	ship->setPosition(p);

	ships.push_back(ship);

	for (unsigned int i=0;i<s.size();i++)
	{
		Sprite* star=new Sprite();
		star->setTexture(resManager.Get(Textures::ID::Minimap_star_1));

		Vector2f p;
		p.x=sprite.getPosition().x+sprite.getLocalBounds().width/2+sprite.getLocalBounds().width*(-player.sprite.getPosition().x+s[i]->sprite.getPosition().x)/15000.0f;
		p.y=sprite.getPosition().y+sprite.getLocalBounds().height/2+sprite.getLocalBounds().height*(-player.sprite.getPosition().y+s[i]->sprite.getPosition().y)/15000.0f;
		star->setPosition(p);

		stars.push_back(star);
	}
}
};

class ContactListener : public b2ContactListener {
public:

void BeginContact(b2Contact *contact) 
{
	if (contact->IsTouching())
	{
	// Box2d objects that collided
	b2Fixture* fixtureA = contact->GetFixtureA();
	b2Fixture* fixtureB = contact->GetFixtureB();

		if ((int)fixtureA->GetBody()->GetUserData()==1)
		{
			Bullet* b=(Bullet*) fixtureA->GetUserData();
			b->toDestroy=true;

			Vector2f p;
			p.x=b->sprite.getPosition().x+15*cos((b->sprite.getRotation()-90.f)*bj_DEGTORAD);
			p.y=b->sprite.getPosition().y+15*sin((b->sprite.getRotation()-90.f)*bj_DEGTORAD);
			booms.push_back(new Boom(resManager,p,500,b->boomType));
		}
		if ((int)fixtureB->GetBody()->GetUserData()==1)
		{
			Bullet* b=(Bullet*) fixtureB->GetUserData();
			b->toDestroy=true;
			//b->sprite.getPosition()
			Vector2f p;
			p.x=b->sprite.getPosition().x+15*cos((b->sprite.getRotation()-90.f)*bj_DEGTORAD);
			p.y=b->sprite.getPosition().y+15*sin((b->sprite.getRotation()-90.f)*bj_DEGTORAD);
			booms.push_back(new Boom(resManager,p,500,b->boomType));
		}

		if ((int)fixtureA->GetBody()->GetUserData()==2&&(int)fixtureB->GetBody()->GetUserData()==4)
		{
			contact->SetEnabled(false);
			Ship* s=(Ship*) fixtureA->GetUserData();
			Mineral* m=(Mineral*) fixtureB->GetUserData();
			m->toDestroy=true;
			s->mineralCount=s->mineralCount+1;
			floatingText.push_back(new FloatingText(resManager,"+1 ",m->sprite.getPosition(),48));
		}
		

	}
}

};

int main()
{
	//load textures
	resManager.Load(Textures::ID::Rocket_1,"Data/Pictures/Rocket_1.png");
	resManager.Load(Textures::ID::Bomb_1,"Data/Pictures/Bomb_1.png");
	resManager.Load(Textures::ID::Laser_1,"Data/Pictures/Laser_1.png");

	resManager.Load(Textures::ID::Boom_1,"Data/Pictures/Boom_1.png");
	resManager.Load(Textures::ID::Boom_laser_1,"Data/Pictures/Boom_laser_1.png");

	resManager.Load(Textures::ID::Ship_1,"Data/Pictures/Ship_1.png");

	resManager.Load(Textures::ID::Menu_button_1,"Data/Pictures/Menu_button.png");
	resManager.Load(Textures::ID::Menu_screen_1,"Data/Pictures/Menu_screen.png");

	resManager.Load(Textures::ID::Menu_new_game,"Data/Pictures/Menu_new_game.png");
	resManager.Load(Textures::ID::Menu_load_game,"Data/Pictures/Menu_load_game.png");
	resManager.Load(Textures::ID::Menu_options_game,"Data/Pictures/Menu_options_game.png");
	resManager.Load(Textures::ID::Menu_about_game,"Data/Pictures/Menu_about_game.png");
	resManager.Load(Textures::ID::Menu_exit_game,"Data/Pictures/Menu_exit_game.png");

	resManager.Load(Textures::ID::Ship_turbo_1,"Data/Pictures/Ship_turbo_1.png");
	resManager.Load(Textures::ID::Ship_turbo_1,"Data/Pictures/Ship_turbo_none.png");

	resManager.Load(Textures::ID::Sky_1,"Data/Pictures/Sky.png");
	resManager.Load(Textures::ID::Message_1,"Data/Pictures/Message_1.png");

	resManager.Load(Textures::ID::Star_light_1,"Data/Pictures/Star_light_1.png");
	resManager.Load(Textures::ID::Star_light_2,"Data/Pictures/Star_light_2.png");
	resManager.Load(Textures::ID::Star_light_3,"Data/Pictures/Star_light_3.png");
	resManager.Load(Textures::ID::Star_light_4,"Data/Pictures/Star_light_4.png");
	resManager.Load(Textures::ID::Star_light_5,"Data/Pictures/Star_light_5.png");

	resManager.Load(Textures::ID::Minimap_1,"Data/Pictures/Minimap_1.png");
	resManager.Load(Textures::ID::Minimap_ship_1,"Data/Pictures/Minimap_ship_1.png");
	resManager.Load(Textures::ID::Minimap_star_1,"Data/Pictures/Minimap_star_1.png");

	resManager.Load(Textures::ID::Planet_1,"Data/Pictures/Planet_1.png");
	resManager.Load(Textures::ID::Planet_2,"Data/Pictures/Planet_2.png");
	resManager.Load(Textures::ID::Planet_3,"Data/Pictures/Planet_3.png");
	resManager.Load(Textures::ID::Planet_4,"Data/Pictures/Planet_4.png");
	resManager.Load(Textures::ID::Planet_5,"Data/Pictures/Planet_5.png");
	resManager.Load(Textures::ID::Planet_6,"Data/Pictures/Planet_6.png");
	resManager.Load(Textures::ID::Planet_7,"Data/Pictures/Planet_7.png");

	resManager.Load(Textures::ID::Mineral_1,"Data/Pictures/Mineral_1.png");
	resManager.Load(Textures::ID::Mineral_2,"Data/Pictures/Mineral_2.png");
	resManager.Load(Textures::ID::Mineral_3,"Data/Pictures/Mineral_3.png");
	resManager.Load(Textures::ID::Mineral_4,"Data/Pictures/Mineral_4.png");
	
	//load sound
	resManager.LoadSound(SoundType::Laser_start_1,"Data/Sound/Laser_start_1.wav");
	resManager.LoadSound(SoundType::Laser_blast_1,"Data/Sound/Laser_blast_1.wav");
	resManager.LoadSound(SoundType::Bomb_blast_1,"Data/Sound/Bomb_blast_1.wav");
	resManager.LoadSound(SoundType::Bomb_start_1,"Data/Sound/Bomb_start_1.wav");
	resManager.LoadSound(SoundType::Rocket_blast_1,"Data/Sound/Rocket_blast_1.wav");
	resManager.LoadSound(SoundType::Rocket_start_1,"Data/Sound/Rocket_start_1.wav");

	resManager.LoadSound(SoundType::Menu_click,"Data/Sound/Menu_click.wav");
	
	//load font
	resManager.LoadFont("Data/arial.ttf");

	srand(time(0));
	b2Vec2 gravity(0.0f, 0.0f);
    b2World world(gravity);
	ContactListener cl;
	world.SetContactListener(&cl);
	Sky sky(resManager,Textures::ID::Sky_1);

	Sprite menuScreen;
	Sprite newGame;
	Sprite loadGame;
	Sprite optionsGame;
	Sprite aboutGame;
	Sprite exitGame;

	Minimap minimap(resManager,Textures::ID::Minimap_1);

	PlanetDecription planetDecription(resManager,Textures::ID::Message_1);

	menuScreen.setTexture(resManager.Get(Textures::ID::Menu_screen_1));
	menuScreen.setPosition(0,0);

	newGame.setTexture(resManager.Get(Textures::ID::Menu_new_game));
	newGame.setPosition(435,270);

	loadGame.setTexture(resManager.Get(Textures::ID::Menu_load_game));
	loadGame.setPosition(435,322);

	optionsGame.setTexture(resManager.Get(Textures::ID::Menu_options_game));
	optionsGame.setPosition(435,374);

	aboutGame.setTexture(resManager.Get(Textures::ID::Menu_about_game));
	aboutGame.setPosition(435,426);

	exitGame.setTexture(resManager.Get(Textures::ID::Menu_exit_game));
	exitGame.setPosition(435,478);


	vector <Planet*> planets;
	vector <Mineral*> minerals;
	Ship player;
	StarLight* starLight=new StarLight(resManager,Textures::ID::Star_light_2,Vector2f(0,0));

	bool gameCreated=false;
	
	vector <Bullet*> bullets;
	vector <Turbo> turbos;

	VideoMode videomode;
	videomode=videomode.getDesktopMode();
	View defaultView;
	View currView;
	currView.reset(FloatRect(0, 0, videomode.width, videomode.height));

	defaultView.reset(FloatRect(0, 0, videomode.width, videomode.height));

	RenderWindow window;
	window.create(videomode, "Ship",Style::Fullscreen);
	window.setView(currView);

	int gameState=GameState::BeforeGame;

	window.setFramerateLimit(60);

	bool isTurbo=false;
	
	Clock clock;
	int time=0;
	int delta;
	bool canFire=true;
while (window.isOpen())
    {
if(gameState==GameState::InGame)
{
	Event event;

	delta=clock.restart().asMilliseconds();
	time=time+delta;
	
	if (time>=100)
	{
		canFire=true;
	}


	//cout<<time<<endl;
	 if (Keyboard::isKeyPressed(Keyboard::Escape))
			{
				gameState=GameState::PauseGame;
			}

	 Vector2f mouse=window.mapPixelToCoords(Mouse::getPosition());
	 bool mouseInPlanet=false;
	 for (unsigned int i=0;i<planets.size();i++)
	 {
		 if (planets[i]->sprite.getGlobalBounds().contains(mouse))
		 {
			 planetDecription.text.setString(planets[i]->description);
			 mouseInPlanet=true;
		 }
	 }

	planetDecription.toShow=mouseInPlanet;


     while (window.pollEvent(event))
        {
			switch(event.type)
			{


			//,   
			case Event::Closed:

				gameState=GameState::ExitGame;
			
			case Event::KeyPressed:
				
				if(event.key.code == sf::Keyboard::Q)
				{
				if (canFire==true)
				{
					time=0;
					canFire=false;

					//fire rocket
					Vector2f point;
					point.x=player.sprite.getPosition().x+40*cos(player.angle);
					point.y=player.sprite.getPosition().y+40*sin(player.angle);
					Bullet* b=new Bullet(resManager,Textures::ID::Rocket_1,world,point.x,point.y,player.angle,15,5,150,BoomType::Rocket_1);
					bullets.push_back(b);

					Sound* s_laser=new Sound(resManager.GetSound(SoundType::Rocket_start_1));
					s_laser->setVolume(50);
					s_laser->play();
					sounds.push_back(s_laser);
				}
				}
				if(event.key.code == sf::Keyboard::E)
				{
				if (canFire==true)
				{

					time=0;
					canFire=false;
					
					//fire bomb
					Vector2f point;
					point.x=player.sprite.getPosition().x+40*cos(player.angle);
					point.y=player.sprite.getPosition().y+40*sin(player.angle);
					Bullet* b=new Bullet(resManager,Textures::ID::Bomb_1,world,point.x,point.y,player.angle,10,10,150,BoomType::Bomb_1);
					bullets.push_back(b);

					Sound* s_laser=new Sound(resManager.GetSound(SoundType::Bomb_start_1));
					s_laser->setVolume(50);
					s_laser->play();
					sounds.push_back(s_laser);
				}
				}

				if(event.key.code == sf::Keyboard::R)
				{
				if (canFire==true)
				{
					time=0;
					canFire=false;
					
					//fire laser
					Vector2f point;
					point.x=player.sprite.getPosition().x+50*cos(player.angle);
					point.y=player.sprite.getPosition().y+50*sin(player.angle);
					Bullet* b=new Bullet(resManager,Textures::ID::Laser_1,world,point.x,point.y,player.angle,12.5f,1.5f,150,BoomType::Laser_1);
					bullets.push_back(b);

					Sound* s_laser=new Sound(resManager.GetSound(SoundType::Laser_start_1));
					s_laser->setVolume(50);
					s_laser->play();
					sounds.push_back(s_laser);

				}
				}
				
                if(event.key.code == sf::Keyboard::W)
				{
					isTurbo=true;
					if (player.direction==2)
					{
						player.body->SetLinearVelocity(b2Vec2(0,0));
						player.direction=0;
						isTurbo=false;
					}
					else
					{
						player.body->SetLinearVelocity(b2Vec2(4*cos(player.angle),4*sin(player.angle)));
						player.direction=1;
					}
					
				}
				if(event.key.code == sf::Keyboard::S)
				{
					isTurbo=false;
					if (player.direction==1)
					{
						player.body->SetLinearVelocity(b2Vec2(0,0));
						player.direction=0;
					
					}
					else
					{
						player.body->SetLinearVelocity(b2Vec2(-4*cos(player.angle),-4*sin(player.angle)));
						player.direction=2;
					}
					
				}
				if(event.key.code == sf::Keyboard::D)
				{
					//player.playerBody->SetAngularVelocity(4.0f);

					player.angle=player.angle+7.5f*bj_DEGTORAD;
						if (player.direction==1)
						{
							isTurbo=true;
							player.body->SetLinearVelocity(b2Vec2(4*cos(player.angle),4*sin(player.angle)));
						}
						else if (player.direction==2)
						{
							isTurbo=true;
							player.body->SetLinearVelocity(b2Vec2(-4*cos(player.angle),-4*sin(player.angle)));
						}
				}
				if(event.key.code == sf::Keyboard::A)
				{
					//player.playerBody->SetAngularVelocity(-4.0f);
					player.angle=player.angle-7.5f*bj_DEGTORAD;
						if (player.direction==1)
						{
							isTurbo=true;
							player.body->SetLinearVelocity(b2Vec2(4*cos(player.angle),4*sin(player.angle)));
						}
						else if (player.direction==2)
						{
							isTurbo=true;
							player.body->SetLinearVelocity(b2Vec2(-4*cos(player.angle),-4*sin(player.angle)));
						}
				}	
					break;

			}
			//endEventLoop

			
        }				

	world.Step(1.0f / 60.0f, 10, 10);
	//
	//
	for (unsigned int i=0;i<bullets.size();i++)
	{
		if (bullets[i]->toDestroy==true||bullets[i]->life==0)
		{

			world.DestroyBody(bullets[i]->body);
			delete bullets[i];
			bullets.erase(bullets.begin()+i);
		}
		else
		{
			bullets[i]->life=bullets[i]->life-1;
			bullets[i]->sprite.setPosition(bullets[i]->body->GetPosition().x*SCALE, bullets[i]->body->GetPosition().y*SCALE);
		}
	}
		for (unsigned int i=0;i<planets.size();i++)
	{
		planets[i]->sprite.setPosition(planets[i]->body->GetPosition().x*SCALE, planets[i]->body->GetPosition().y*SCALE);
	}
	for (unsigned int i=0;i<minerals.size();i++)
	{
		if (minerals[i]->toDestroy==true)
		{
			world.DestroyBody(minerals[i]->body);
			delete minerals[i];
			minerals.erase(minerals.begin()+i);
		}
		else
		{
			minerals[i]->sprite.setPosition(minerals[i]->body->GetPosition().x*SCALE, minerals[i]->body->GetPosition().y*SCALE);
			minerals[i]->sprite.setRotation(minerals[i]->body->GetAngle()*bj_RADTODEG);
		}
	}

	for (unsigned int i=0;i<floatingText.size();i++)
	{
		if (floatingText[i]->Update()==true)
		{
			delete floatingText[i];
			floatingText.erase(floatingText.begin()+i);
		}
	}

	for (unsigned int i=0;i<booms.size();i++)
	{
		if (booms[i]->Update(delta)==true)
		{
			delete booms[i];
			booms.erase(booms.begin()+i);
		}
		else
		{
			booms[i]->anim->Update(delta);
		}
	}

	for (unsigned int i=0;i<bullets.size();i++)
	{
		bullets[i]->sprite.setPosition(bullets[i]->body->GetPosition().x*SCALE, bullets[i]->body->GetPosition().y*SCALE);
	}

	player.oldPosition=player.sprite.getPosition();

	player.sprite.setPosition((player.body->GetPosition().x)*SCALE, (player.body->GetPosition().y)*SCALE);
	
	//****
	//***
	starLight->anim->sprite.setPosition(player.sprite.getPosition().x-50,player.sprite.getPosition().y-50);
	//starLight->anim->Update(delta);

	currView.setCenter(player.sprite.getPosition());
	// skyView.move((player.sprite.getPosition().x-player.oldPosition.x)*1.5,(player.sprite.getPosition().y-player.oldPosition.y)*1.5);
	 sky.sprite.setPosition(player.sprite.getPosition());

	 //player.angle=atan2(mouse.y-player.sprite.getPosition().y,mouse.x-player.sprite.getPosition().x);
	 player.body->SetTransform(player.body->GetPosition(),player.angle+90.0f*bj_DEGTORAD);
	 player.sprite.setRotation(player.angle*bj_RADTODEG+90.0f);

	minimap.Update(resManager,player,planets);
	
	window.setView(currView);
	window.clear();
	window.draw(sky.sprite);
	window.draw(player.sprite);

	window.draw(starLight->anim->sprite);
		

	for (unsigned int i=0;i<bullets.size();i++)
	{
		window.draw(bullets[i]->sprite);
	}

	for (unsigned int i=0;i<planets.size();i++)
	{
		window.draw(planets[i]->sprite);
	}

	for (unsigned int i=0;i<minerals.size();i++)
	{
		window.draw(minerals[i]->sprite);
	}

	for (unsigned int i=0;i<booms.size();i++)
	{
		window.draw(booms[i]->anim->sprite);
	}

	for (unsigned int i=0;i<floatingText.size();i++)
	{
		window.draw(floatingText[i]->text);
	}

	window.setView(defaultView);
	window.draw(minimap.sprite);

	for (unsigned int i=0;i<minimap.stars.size();i++)
	{
		window.draw(*minimap.stars[i]);
	}

	for (unsigned int i=0;i<minimap.ships.size();i++)
	{
		window.draw(*minimap.ships[i]);
	}

	window.draw(planetDecription.sprite);

	if (planetDecription.toShow==true)
	{
		window.draw(planetDecription.text);
	}

	for (unsigned int i=0;i<sounds.size();i++)
	{
		if (sounds[i]->getStatus()==Sound::Status::Stopped)
		{
			delete sounds[i];
			sounds.erase(sounds.begin()+i);
		}
	}

	player.text.setString(" : "+Int2String(player.mineralCount));
	window.draw(player.text);
	window.setView(currView);

	window.display();

}//end if inGame
else if (gameState==GameState::PauseGame)
{
	bool isPaused=true;
	Event event;
    while (window.pollEvent(event))
     {
		 if(event.key.code == sf::Mouse::Left)
	{
		Vector2i mp=Mouse::getPosition();
		if (gameCreated==false)
		{
		if (newGame.getGlobalBounds().contains(mp.x,mp.y))
		{
	
			isPaused=false;
			gameState=GameState::InGame;
		}
		}
		if (loadGame.getGlobalBounds().contains(mp.x,mp.y))
		{
			isPaused=false;
			gameState=GameState::InGame;
		}
		if (optionsGame.getGlobalBounds().contains(mp.x,mp.y))
		{
			isPaused=false;
			gameState=GameState::InGame;
		}
		if (aboutGame.getGlobalBounds().contains(mp.x,mp.y))
		{
			isPaused=false;
			gameState=GameState::InGame;
		}
		if (exitGame.getGlobalBounds().contains(mp.x,mp.y))
		{
			gameState=GameState::ExitGame;
		}
	}
	 }
	if (isPaused==true)
	{
	window.clear();
	window.setView(defaultView);
	window.draw(menuScreen);
	if (gameCreated==false)
	{
	window.draw(newGame);
	}
	window.draw(loadGame);
	window.draw(optionsGame);
	window.draw(aboutGame);
	window.draw(exitGame);
	window.display();
	}
}//end if pauseGame

else if (gameState==GameState::ExitGame)
{
	window.setMouseCursorVisible(true);
    window.close();
}//end if exitGame

else if (gameState==GameState::OptionsGame)
{
}//end if optionsGame

else if (gameState==GameState::AboutGame)
{
}//end if aboutGame

else if (gameState==GameState::BeforeGame)
{
	bool toCreate=false;
	if (gameCreated==false)
	{
	Event event;
    while (window.pollEvent(event))
     {
		 if(event.key.code == sf::Mouse::Left)
	{
		Vector2i mp=Mouse::getPosition();
		if (newGame.getGlobalBounds().contains(mp.x,mp.y))
		{
			toCreate=true;
		}
		if (loadGame.getGlobalBounds().contains(mp.x,mp.y))
		{
			toCreate=true;
		}
		if (optionsGame.getGlobalBounds().contains(mp.x,mp.y))
		{
			toCreate=true;
		}
		if (aboutGame.getGlobalBounds().contains(mp.x,mp.y))
		{
			toCreate=true;
		}
		if (exitGame.getGlobalBounds().contains(mp.x,mp.y))
		{
			gameState=GameState::ExitGame;
		}
	}
	 }
	}
	if (toCreate==true)
	{
		Sound* s_laser=new Sound(resManager.GetSound(SoundType::Menu_click));
		s_laser->setVolume(50);
		s_laser->play();
		sounds.push_back(s_laser);
	gameCreated=true;

	int x1=GetRandomInt(0,15000);
	int y1=GetRandomInt(0,15000);
	player.Create(resManager,Textures::ID::Ship_1,world,x1+300,y1+300,90*bj_DEGTORAD);

	int mult=100;
	
	int baseAngle=(GetRandomInt(0,360))*bj_DEGTORAD;
	int planetCount=GetRandomInt(3,5);
	string s;
	s=" : \n : ";
	planets.push_back(new Planet(resManager,Textures::ID::Planet_2,world,x1,y1,s));

	for (unsigned int i=0;i<planetCount;i++)
	{
		int random=GetRandomInt(1,4);
		int angle=360/planetCount*i*bj_DEGTORAD;
		int distance=GetRandomInt(400,866);
				if (random==1)
				{
					s=" : \n : \n   : "+Int2String(distance);
					planets.push_back(new Planet(resManager,Textures::ID::Planet_3,world,x1+distance*cos(angle),y1+distance*sin(angle),s));
				}	
				else if (random==2)
				{
					s=" : \n :  \n   : "+Int2String(distance);
					planets.push_back(new Planet(resManager,Textures::ID::Planet_4,world,x1+distance*cos(angle),y1+distance*sin(angle),s));
				}	
				else if (random==3)
				{
					s=" : \n :  \n   : "+Int2String(distance);
					planets.push_back(new Planet(resManager,Textures::ID::Planet_5,world,x1+distance*cos(angle),y1+distance*sin(angle),s));
				}	
				else if (random==4)
				{
					s=" : \n : \n   : "+Int2String(distance);
					planets.push_back(new Planet(resManager,Textures::ID::Planet_7,world,x1+distance*cos(angle),y1+distance*sin(angle),s));
				}	

				random=GetRandomInt(1,4);
				if (random==1)
				{				
					int rangle=(GetRandomInt(0,360))*bj_DEGTORAD;
					int rdist=GetRandomInt(120,150);
					s=" : \n : \n   : "+Int2String(rdist);
					planets.push_back(new Planet(resManager,Textures::ID::Planet_1,world,x1+distance*cos(angle)+rdist*cos(rangle),y1+distance*sin(angle)+rdist*sin(rangle),s));
				}
				else if (random==2)
				{				
					int rangle=(GetRandomInt(0,360))*bj_DEGTORAD;
					int rdist=GetRandomInt(120,150);
					s=" : \n : \n   : "+Int2String(rdist);
					planets.push_back(new Planet(resManager,Textures::ID::Planet_6,world,x1+distance*cos(angle)+rdist*cos(rangle),y1+distance*sin(angle)+rdist*sin(rangle),s));
				}
				int mineralCount=GetRandomInt(3,6);
					for (unsigned int j=0;j<mineralCount;j++)
					{
						int mineralNumber=GetRandomInt(1,4);
						int rangle=360/mineralCount*j*bj_DEGTORAD;
						int rdist=GetRandomInt(160,200);
						if (mineralNumber==1)
						{
						minerals.push_back(new Mineral(resManager,Textures::ID::Mineral_1,world,x1+distance*cos(angle)+rdist*cos(rangle),y1+distance*sin(angle)+rdist*sin(rangle)));
						}
						else if (mineralNumber==2)
						{
						minerals.push_back(new Mineral(resManager,Textures::ID::Mineral_2,world,x1+distance*cos(angle)+rdist*cos(rangle),y1+distance*sin(angle)+rdist*sin(rangle)));
						}
						else if (mineralNumber==3)
						{
						minerals.push_back(new Mineral(resManager,Textures::ID::Mineral_3,world,x1+distance*cos(angle)+rdist*cos(rangle),y1+distance*sin(angle)+rdist*sin(rangle)));
						}
						else if (mineralNumber==4)
						{
						minerals.push_back(new Mineral(resManager,Textures::ID::Mineral_4,world,x1+distance*cos(angle)+rdist*cos(rangle),y1+distance*sin(angle)+rdist*sin(rangle)));
						}

					}
	}

	gameState=GameState::InGame;
	}
	else
	{
		
	window.clear();
	window.setView(defaultView);
	window.draw(menuScreen);
	window.draw(newGame);
	window.draw(loadGame);
	window.draw(optionsGame);
	window.draw(aboutGame);
	window.draw(exitGame);
	window.display();

	}
}//end if beforeGame

//end of state check
}//end loop
return 0;
}
